export declare type Validator<T> = {
    [key in keyof T]: (arg: any) => T[key];
};
export declare type Validated<T> = {
    [key in keyof T]: T[key];
};
export interface ILength {
    length: number;
}
export declare abstract class PathNode {
}
export declare class KeyPathNode extends PathNode {
    readonly key: string;
    constructor(key: string);
    toString(): string;
}
export declare class ArrayIndexPathNode extends PathNode {
    readonly index: number;
    constructor(index: number);
    toString(): string;
}
export declare class ValidationError {
    readonly errorCode: string;
    readonly message: string;
    readonly path: PathNode[];
    constructor(errorCode: string, message: string);
    toString(root?: string): string;
    pathString(root?: string): string;
}
export declare class ValidationErrorCollection {
    readonly errors: ValidationError[];
    constructor(error?: ValidationError);
    insertError(node: PathNode, error: ValidationError): void;
    handleError(node: PathNode, err: any): void;
    toString(root?: string): string;
}
export interface IValidationOptions {
    allowAdditionalProperties?: boolean;
}
export declare function validate<T>(arg: any, validator: Validator<T>, options?: IValidationOptions): Validated<T>;
export declare function extendValidator<T, U>(validator1: Validator<T>, validator2: Validator<U>): Validator<T & U>;
export declare function optional<T>(next: (arg: any) => T): (arg: any) => T | undefined;
export declare function nullable<T>(next: (arg: any) => T): (arg: any) => T | null;
export declare function defaultsTo(def: any): (arg: any) => any;
export declare function defaultsTo<T>(def: T, next: (arg: any) => T): (arg: any) => T;
export declare function onErrorDefaultsTo<T, U>(def: U, next: (arg: T) => U): (arg: T) => U;
export declare function isBoolean(): (arg: any) => boolean;
export declare function isBoolean<T = boolean>(next: (arg: boolean) => T): (arg: any) => T;
export declare function isNumber(): (arg: any) => number;
export declare function isNumber<T = number>(next: (arg: number) => T): (arg: any) => T;
export declare function min(min: number): (arg: number) => number;
export declare function min<T = number>(min: number, next: (arg: number) => T): (arg: number) => T;
export declare function max(max: number): (arg: number) => number;
export declare function max<T = number>(max: number, next: (arg: number) => T): (arg: number) => T;
export declare function isString(): (arg: any) => string;
export declare function isString<T = string>(next: (arg: string) => T): (arg: any) => T;
export declare function matches(regex: RegExp): (arg: string) => string;
export declare function matches<T = string>(regex: RegExp, next: (arg: string) => T): (arg: string) => T;
export declare function minLength<T extends ILength>(min: number): (arg: T) => T;
export declare function minLength<T extends ILength, U = T>(min: number, next: (arg: T) => U): (arg: T) => U;
export declare function maxLength<T extends ILength>(max: number): (arg: T) => T;
export declare function maxLength<T extends ILength, U = T>(max: number, next: (arg: T) => U): (arg: T) => U;
export declare function lengthIs<T extends ILength>(length: number): (arg: T) => T;
export declare function lengthIs<T extends ILength, U = T>(length: number, next: (arg: T) => U): (arg: T) => U;
export declare function isArray(): (arg: any) => any[];
export declare function isArray<T>(next: (arg: any[]) => T): (arg: any) => T;
export declare function eachItem<T>(assertion: (arg: any) => T): (arg: any[]) => T[];
export declare function eachItem<T, U>(assertion: (arg: any) => T, next: (arg: T[]) => U): (arg: any[]) => U;
export declare function isObject(): (arg: any) => any;
export declare function isObject<T>(next: (arg: any) => T): (arg: any) => T;
export declare function conformsTo<T>(validator: Validator<T>): (arg: any) => Validated<T>;
export declare function conformsTo<T>(validator: Validator<T>, options: IValidationOptions): (arg: any) => Validated<T>;
export declare function conformsTo<T, U>(validator: Validator<T>, next: (arg: Validated<T>) => U): (arg: any) => U;
export declare function conformsTo<T, U>(validator: Validator<T>, options: IValidationOptions, next: (arg: Validated<T>) => U): (arg: any) => U;
export declare function equals<T>(value: T, ...values: T[]): (arg: any) => T;
