"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var PathNode = /** @class */ (function () {
    function PathNode() {
    }
    return PathNode;
}());
exports.PathNode = PathNode;
var KeyPathNode = /** @class */ (function (_super) {
    __extends(KeyPathNode, _super);
    function KeyPathNode(key) {
        var _this = _super.call(this) || this;
        _this.key = key;
        return _this;
    }
    KeyPathNode.prototype.toString = function () {
        if (/^[$a-z_][$a-z0-9_]*$/i.test(this.key)) {
            return "." + this.key;
        }
        else {
            return "['" + this.key.replace('\\', '\\\\').replace("'", "\\'") + "']";
        }
    };
    return KeyPathNode;
}(PathNode));
exports.KeyPathNode = KeyPathNode;
var ArrayIndexPathNode = /** @class */ (function (_super) {
    __extends(ArrayIndexPathNode, _super);
    function ArrayIndexPathNode(index) {
        var _this = _super.call(this) || this;
        _this.index = index;
        return _this;
    }
    ArrayIndexPathNode.prototype.toString = function () {
        return "[" + this.index + "]";
    };
    return ArrayIndexPathNode;
}(PathNode));
exports.ArrayIndexPathNode = ArrayIndexPathNode;
var ValidationError = /** @class */ (function () {
    function ValidationError(errorCode, message) {
        this.errorCode = errorCode;
        this.message = message;
        this.path = [];
    }
    ValidationError.prototype.toString = function (root) {
        if (root === void 0) { root = '$root'; }
        return this.pathString(root) + ": " + this.message;
    };
    ValidationError.prototype.pathString = function (root) {
        if (root === void 0) { root = '$root'; }
        return root + this.path.map(function (node) { return node.toString(); }).join('');
    };
    return ValidationError;
}());
exports.ValidationError = ValidationError;
var ValidationErrorCollection = /** @class */ (function () {
    function ValidationErrorCollection(error) {
        this.errors = [];
        if (error) {
            this.errors.push(error);
        }
    }
    ValidationErrorCollection.prototype.insertError = function (node, error) {
        error.path.unshift(node);
        this.errors.push(error);
    };
    ValidationErrorCollection.prototype.handleError = function (node, err) {
        if (err instanceof ValidationErrorCollection) {
            for (var _i = 0, _a = err.errors; _i < _a.length; _i++) {
                var error = _a[_i];
                this.insertError(node, error);
            }
        }
        else if (err instanceof ValidationError) {
            this.insertError(node, err);
        }
        else {
            this.insertError(node, new ValidationError('UNHANDLED_ERROR', "" + (typeof err === 'object' && err.message || 'Unknown error')));
        }
    };
    ValidationErrorCollection.prototype.toString = function (root) {
        if (root === void 0) { root = '$root'; }
        return this.errors.length + " validation error" + (this.errors.length === 1 ? '' : 's') + ":\n  " + this.errors.map(function (error) { return error.toString(root); }).join('\n  ');
    };
    return ValidationErrorCollection;
}());
exports.ValidationErrorCollection = ValidationErrorCollection;
function validate(arg, validator, options) {
    if (options === void 0) { options = {}; }
    var _a = options.allowAdditionalProperties, allowAdditionalProperties = _a === void 0 ? true : _a;
    if (typeof arg !== 'object')
        throw new ValidationErrorCollection(new ValidationError('NOT_OBJECT', "Expected object, got " + typeof arg));
    var result = {};
    var validationErrorCollection = null;
    var validatedProperties = Object.keys(arg).reduce(function (validatedProperties, key) {
        validatedProperties[key] = false;
        return validatedProperties;
    }, {});
    for (var key in validator) {
        validatedProperties[key] = true;
        try {
            result[key] = validator[key](arg[key]);
        }
        catch (err) {
            if (validationErrorCollection === null) {
                validationErrorCollection = new ValidationErrorCollection();
            }
            validationErrorCollection.handleError(new KeyPathNode(key), err);
        }
    }
    if (!allowAdditionalProperties && !Object.keys(validatedProperties).every(function (key) { return validatedProperties[key]; })) {
        if (validationErrorCollection === null) {
            validationErrorCollection = new ValidationErrorCollection();
        }
        validationErrorCollection.errors.push(new ValidationError('UNEXPECTED_ADDITIONAL_PROPERTIES', "Unexpected additional properties: " + Object.keys(validatedProperties).filter(function (key) { return !validatedProperties[key]; }).join(', ')));
    }
    if (validationErrorCollection !== null)
        throw validationErrorCollection;
    return result;
}
exports.validate = validate;
function extendValidator(validator1, validator2) {
    var result = {};
    for (var key in validator1) {
        result[key] = validator1[key];
    }
    for (var key in validator2) {
        result[key] = validator2[key];
    }
    return result;
}
exports.extendValidator = extendValidator;
// ASSERTIONS //
function optional(next) {
    return function (arg) {
        if (arg === undefined)
            return undefined;
        return next(arg);
    };
}
exports.optional = optional;
function nullable(next) {
    return function (arg) {
        if (arg === null)
            return null;
        return next(arg);
    };
}
exports.nullable = nullable;
function defaultsTo(def, next) {
    return function (arg) {
        if (arg === undefined)
            arg = def;
        return next ? next(arg) : arg;
    };
}
exports.defaultsTo = defaultsTo;
function onErrorDefaultsTo(def, next) {
    return function (arg) {
        try {
            return next(arg);
        }
        catch (_) {
            // Ignore error - resort to default
            return def;
        }
    };
}
exports.onErrorDefaultsTo = onErrorDefaultsTo;
function isBoolean(next) {
    return function (arg) {
        if (typeof arg !== 'boolean')
            throw new ValidationError('NOT_BOOLEAN', "Expected boolean, got " + typeof arg);
        return next ? next(arg) : arg;
    };
}
exports.isBoolean = isBoolean;
function isNumber(next) {
    return function (arg) {
        if (typeof arg !== 'number')
            throw new ValidationError('NOT_NUMBER', "Expected number, got " + typeof arg);
        return next ? next(arg) : arg;
    };
}
exports.isNumber = isNumber;
function min(min, next) {
    return function (arg) {
        if (arg < min)
            throw new ValidationError('LESS_THAN_MIN', arg + " is less than " + min);
        return next ? next(arg) : arg;
    };
}
exports.min = min;
function max(max, next) {
    return function (arg) {
        if (arg > max)
            throw new ValidationError('GREATER_THAN_MAX', arg + " is greater than " + max);
        return next ? next(arg) : arg;
    };
}
exports.max = max;
function isString(next) {
    return function (arg) {
        if (typeof arg !== 'string')
            throw new ValidationError('NOT_STRING', "Expected string, got " + typeof arg);
        return next ? next(arg) : arg;
    };
}
exports.isString = isString;
function matches(regex, next) {
    return function (arg) {
        if (!regex.test(arg))
            throw new ValidationError('FAILED_REGEXP', "Failed regular expression " + regex.toString());
        return next ? next(arg) : arg;
    };
}
exports.matches = matches;
function minLength(min, next) {
    return function (arg) {
        if (arg.length < min)
            throw new ValidationError('LESS_THAN_MIN_LENGTH', "Length " + arg.length + " is less than " + min);
        return next ? next(arg) : arg;
    };
}
exports.minLength = minLength;
function maxLength(max, next) {
    return function (arg) {
        if (arg.length > max)
            throw new ValidationError('GREATER_THAN_MAX_LENGTH', "Length " + arg.length + " is greater than " + max);
        return next ? next(arg) : arg;
    };
}
exports.maxLength = maxLength;
function lengthIs(length, next) {
    return function (arg) {
        if (arg.length !== length)
            throw new ValidationError('LENGTH_NOT_EQUAL', "Length " + arg.length + " is not equal to " + length);
        return next ? next(arg) : arg;
    };
}
exports.lengthIs = lengthIs;
function isArray(next) {
    return function (arg) {
        if (!(arg instanceof Array))
            throw new ValidationError('NOT_ARRAY', "Expected array, got " + typeof arg);
        return next ? next(arg) : arg;
    };
}
exports.isArray = isArray;
function eachItem(assertion, next) {
    return function (arg) {
        var validationErrorCollection = null;
        var mapped = arg.map(function (item, index) {
            try {
                return assertion(item);
            }
            catch (err) {
                if (validationErrorCollection === null) {
                    validationErrorCollection = new ValidationErrorCollection();
                }
                validationErrorCollection.handleError(new ArrayIndexPathNode(index), err);
            }
        });
        if (validationErrorCollection !== null)
            throw validationErrorCollection;
        return next ? next(mapped) : mapped;
    };
}
exports.eachItem = eachItem;
function isObject(next) {
    return function (arg) {
        if (typeof arg !== 'object')
            throw new ValidationError('NOT_OBJECT', "Expected object, got " + typeof arg);
        return next ? next(arg) : arg;
    };
}
exports.isObject = isObject;
function conformsTo(validator, optionsOrNext, next) {
    return function (arg) {
        if (typeof optionsOrNext === 'function') {
            next = optionsOrNext;
        }
        var validated = validate(arg, validator, typeof optionsOrNext === 'object' ? optionsOrNext : undefined);
        return next ? next(validated) : validated;
    };
}
exports.conformsTo = conformsTo;
function equals(value) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    var vals = [value].concat(values);
    return function (arg) {
        for (var _i = 0, vals_1 = vals; _i < vals_1.length; _i++) {
            var val = vals_1[_i];
            if (val === arg)
                return arg;
        }
        throw new ValidationError('NOT_EQUAL', vals.length === 1 ? "'" + arg + "' does not equal '" + vals[0] + "'" : "'" + arg + "' not one of: " + vals.join(', '));
    };
}
exports.equals = equals;
//# sourceMappingURL=index.js.map